package lsedit;

import java.awt.Component;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;

public class TextTree extends JTree implements MouseListener
{
	public    static final String m_helpStr	 = "Right click for menu.";

	protected static final Color titleColor = Color.red.darker();

	protected static final Font m_titleFont = new Font("Helvetica", Font.BOLD, 14);
	protected static final Font m_textFont =	new Font("Helvetica", Font.PLAIN, 11); 

	protected LandscapeViewerCore m_ls;
	protected JTabbedPane		  m_tabbedPane;
	protected JScrollPane		  m_scrollPane;
	protected JPopupMenu		  m_popupMenu;

	// --------------
	// Public methods 
	// --------------

	public TextTree(LandscapeViewerCore ls, JTabbedPane tabbedPane) 
	{
		super();

		DefaultTreeCellRenderer	renderer;

		m_ls          = ls;
		m_tabbedPane  = tabbedPane;

		setBackground(Diagram.boxColour);
		setFont(m_textFont);
		setEditable(false);
		setShowsRootHandles(true);
		renderer = (DefaultTreeCellRenderer) getCellRenderer();
		renderer.setBackgroundNonSelectionColor(Diagram.boxColour);
//		renderer.setBackgroundSelectionColor(Color.BLUE); 
		setCellRenderer(renderer);

		setToolTipText(m_helpStr);
		getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

		m_scrollPane = new JScrollPane();
		m_scrollPane.setBounds(tabbedPane.getBounds());
		setBounds(tabbedPane.getBounds());
		m_scrollPane.setViewportView(this);

		tabbedPane.addTab("TOC", null, m_scrollPane, m_helpStr);
		addMouseListener(this);
	}
	
	// Special over-ride because a JTree thinks the tool tip is associated only with cells being displayed

	public String getToolTipText(MouseEvent event) 
	{	
		String ret = super.getToolTipText(event);

		if (ret == null) {
			ret = m_helpStr;
		}
		return(ret);
	}

	protected JPopupMenu buildPopup()
	{
		JPopupMenu m;
		
		m = new JPopupMenu("TOC options");

		Do.tocMenuItem(m, m_ls);
		return(m);
	}

	public void closeAll()
	{
		int	i, cnt;

		cnt = getRowCount();
		for (i = cnt; i > 0; ) {
			collapseRow(--i);
	}	}

	public void expandAll()
	{
		int	cnt;

		for (cnt = 0; cnt < getRowCount(); ++cnt) {
			expandRow(cnt);
	}	}

	public void expandTo(EntityInstance e)
	{
		int				cnt;
		TreePath		path;
		TreeNode		treeNode;
		EntityInstance	e1;

		closeAll();
		clearSelection();
		if (e != null) {
			for (cnt = 0; cnt < getRowCount(); ++cnt) {
				path     = getPathForRow(cnt);
				treeNode = (TreeNode) path.getLastPathComponent();
				if (treeNode != null) {
					e1 = (EntityInstance) ((DefaultMutableTreeNode) treeNode).getUserObject();
					if (e1 != null) {
						if (e1 == e) {
							setSelectionRow(cnt);
							scrollRowToVisible(cnt);
							break;
						}
						if (e1.hasDescendant(e)) {
							expandRow(cnt);
	}	}	}	}	}	} 
 
 	public void activate() 
	{
		m_tabbedPane.setSelectedComponent(m_scrollPane);
	}

	public boolean isActive() 
	{
		Component active;

		if (!isVisible()) {
			return(false);
		}
		active = m_tabbedPane.getSelectedComponent();
		return(active != null && this == active);
	}

	// Generic 	MouseListener interface

	public void mouseClicked(MouseEvent ev)
	{
		if (ev.isMetaDown()) {
			int				x, y;
			TreePath		selPath;
			TreeNode		treeNode;
			EntityInstance	e;
			Diagram			diagram;
					
			x       = ev.getX();
			y       = ev.getY();
			selPath = getPathForLocation(x, y);
			if(selPath == null) {
				if (m_popupMenu == null) {
					m_popupMenu = buildPopup();
				}
				add(m_popupMenu);
				m_popupMenu.show(this, x, y);
			} else {
				treeNode = (TreeNode) selPath.getLastPathComponent();
				if (treeNode != null) {
					e = (EntityInstance) ((DefaultMutableTreeNode) treeNode).getUserObject();
					if (e != null) {
						diagram = m_ls.getDiagram();
						if (diagram != null) {
							diagram.navigateTo(e);
		}	}	}	}	}
	}

	public void mouseEntered(MouseEvent ev)
	{
	}

	public void mouseExited(MouseEvent e)
	{
	}

	public void mousePressed(MouseEvent ev)
	{
	}

	public void mouseReleased(MouseEvent ev)
	{
	}



}





